//+------------------------------------------------------------------+
//|                                                        Bands.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2013, Filon"
#property link    "http://www.metaquotes.net"

#property indicator_chart_window
#property indicator_buffers 4
#property indicator_color1 Aqua
#property indicator_color2 DeepSkyBlue
#property indicator_color3 Red
//---- indicator parameters------------------------------------------
extern int    BandsPeriod=30;
extern int    BandsShift=0;
extern double BandsDeviations=2.618;
extern double MovingAverage=MODE_EMA;
extern double Dig      =4;                           //   
extern color Color_Up =DeepSkyBlue;                  //  +
extern color Color_Dn =Tomato;                       //  -
extern color Color_0  =Lime;                         //  0
string Obj_Name ="";                    //  
color  Color_Day_Percent;                            //   
//---- buffers------------------------------------------------------
double MovingBuffer[];
double UpperBuffer[];
double LowerBuffer[];
double deviationBuffer[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicators---------------------------------------------------
   SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(0,MovingBuffer);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexBuffer(1,UpperBuffer);
   SetIndexStyle(2,DRAW_LINE);
   SetIndexBuffer(2,LowerBuffer);
   SetIndexStyle(3,DRAW_LINE);
   SetIndexBuffer(3,deviationBuffer);
//--------------------------------------------------------------------
   SetIndexDrawBegin(0,BandsPeriod+BandsShift);
   SetIndexDrawBegin(1,BandsPeriod+BandsShift);
   SetIndexDrawBegin(2,BandsPeriod+BandsShift);
//-------------------------------------------------------------------
   return(0);
  }
//+------------------------------------------------------------------+
//| Bollinger Bands                                                  |
//+------------------------------------------------------------------+
int start()
  {
   int    i,k,counted_bars=IndicatorCounted();
   double sum,oldval,deviation,newres;
//--------------------------------------------------------------------
   if(Bars<=BandsPeriod) return(0);
//---- initial zero-----------------------------------------------------
   if(counted_bars<1)
      for(i=1;i<=BandsPeriod;i++)
        {
         MovingBuffer[Bars-i]=EMPTY_VALUE;
         UpperBuffer[Bars-i]=EMPTY_VALUE;
         LowerBuffer[Bars-i]=EMPTY_VALUE;
        }
//--------------------------------------------------------------------
   int limit=Bars-counted_bars;
   if(counted_bars>0) limit++;
   for(i=0; i<limit; i++)
      MovingBuffer[i]=iMA(NULL,0,BandsPeriod,BandsShift,MovingAverage,PRICE_CLOSE,i);
//--------------------------------------------------------------------
   i=Bars-BandsPeriod+1;
   if(counted_bars>BandsPeriod-1) i=Bars-counted_bars-1;
   while(i>=0)
     {
      sum=0.0;
      k=i+BandsPeriod-1;
      oldval=MovingBuffer[i];
      while(k>=i)
        {
         newres=Close[k]-oldval;
         sum+=newres*newres;
         k--;
        }
      deviation=BandsDeviations*MathSqrt(sum/BandsPeriod);
      UpperBuffer[i]=oldval+deviation;
      LowerBuffer[i]=oldval-deviation;
///-------------------------------------------------------------------
      string Text =DoubleToStr( deviation, Dig);               //   
      if(NormalizeDouble(deviation,Dig)>deviation-i)           //     
      Color_Day_Percent = Color_Up;                            //   +
      if(NormalizeDouble(deviation,Dig)<deviation-i)           //     
      Color_Day_Percent = Color_Dn;                            //   -
      if(NormalizeDouble(deviation,Dig)==deviation-i)          //     
      Color_Day_Percent = Color_0;                             //   0
      if(ObjectFind(Obj_Name)==-1)                             //   ..
      {
      ObjectCreate(Obj_Name,OBJ_LABEL,0,0,0);                  //    
      ObjectSet    (Obj_Name,OBJPROP_CORNER,    1);            //              
      ObjectSet    (Obj_Name,OBJPROP_XDISTANCE,15);            //  
      ObjectSet    (Obj_Name,OBJPROP_YDISTANCE,15);            //  Y
      }
      ObjectSetText(Obj_Name,Text,10,"MS Sans Serif",Color_Day_Percent);//         
      i--;
     }
//----------------------------------------------------------------------
   return(0);
  }
int deinit()
   {
   ObjectDelete(Obj_Name);                                      //    
   return(0);
   }
//+------------------------------------------------------------------+